-- Durch diese Prozedur knnen Rechte pauschal an mehrere Objekte vergeben werden
-- Aufruf wre dann in etwa so:
-- Alle Berechtigungen fr Rolle auf Tabellen entziehen
-- sp__SetAllPermissions 'REVOKE' , 'TABLE', 'Rolle', 'ALL'
-- Lschen verweigern fr Rolle auf Tabellen
-- sp__SetAllPermissions 'DENY', 'TABLE', 'Rolle', 'DELETE'
-- Rolle public Ausfhrungsberichtung auf allen Tabellen erteilen
-- Execute kann entfallen, da ohnehin einziges mgliches Recht
-- sp__SetAllPermissions 'GRANT' , 'PROC', 'public'
USE master
GO
DROP PROCEDURE sp__SetAllPermissions
GO

CREATE PROCEDURE sp__SetAllPermissions
@grantrevoke nvarchar(6) = NULL, -- GRANT, REVOKE, DENY
@objecttype nvarchar(6) = NULL, -- Objekttyp: U = Tabelle, V = View, P = Procedure
@account sysname = NULL, -- Gruppe oder Benutzer (TO)
@permission nvarchar(8) = NULL -- ALL, SELECT, INSERT, UPDATE, DELETE, EXECUTE
AS
SET NOCOUNT ON

-- CHECK PERMISSIONS
IF (NOT IS_MEMBER('db_securityadmin') = 1) AND (NOT IS_MEMBER('db_owner') = 1)
BEGIN
RAISERROR( 15000, -1, -1)
RETURN (1)
END

IF @grantrevoke = ''
SET @grantrevoke = NULL
ELSE
SET @grantrevoke = UPPER( @grantrevoke )
IF @grantrevoke IS NULL OR @grantrevoke NOT IN ('GRANT', 'REVOKE','DENY')
BEGIN
RAISERROR( 'Geben Sie den Berechtigungtyp an: ''GRANT'', ''REVOKE'',''DENY''', 16, -1 )
RETURN -1
END

IF @objecttype = ''
SET @objecttype = NULL
ELSE
SET @objecttype =
CASE UPPER( @objecttype )
WHEN 'TABLE' THEN 'U'
WHEN 'VIEW' THEN 'V'
WHEN 'PROC' THEN 'P'
ELSE UPPER( @objecttype) END
IF @objecttype IS NULL OR @objecttype NOT IN ('U', 'V', 'P')
BEGIN
RAISERROR( 'Geben Sie den Objekttypen an: ''U''=Tabelle,''V''=Sicht, ''P''=Prozedur', 16, -1 )
RETURN -1
END

IF @account = ''
SET @account = NULL
IF @account IS NULL
BEGIN
RAISERROR( 'Geben Sie den Kontonamen an.', 16, -1 )
RETURN -1
END

IF @permission = ''
SET @permission = NULL
ELSE
SET @permission = UPPER( @permission )
IF @permission IS NULL
BEGIN
-- Hier Standardberechtigungen (restriktiv!)
IF @objecttype IN ('U', 'V')
BEGIN
IF @grantrevoke = 'GRANT'
SET @permission = 'SELECT'
ELSE
SET @permission = 'ALL'
END ELSE BEGIN
SET @permission = 'EXECUTE'
END
END ELSE BEGIN
IF @objecttype IN ('U', 'V')
BEGIN
IF NOT (@permission IN ( 'SELECT', 'INSERT', 'UPDATE', 'DELETE','ALL' ) )
BEGIN
RAISERROR( 'Fr Tabellen/Sichten sind nur die Berechtigungen
''SELECT'', ''INSERT'', ''UPDATE'', ''DELETE'', ''ALL'' erlaubt', 16, -1 )
RETURN -1
END
END ELSE BEGIN
IF NOT (@permission IN ( 'EXECUTE' ) )
BEGIN
RAISERROR( 'Fr Prozeduren ist nur die Berechtigung
''EXECUTE'' erlaubt', 16, -1 )
RETURN -1
END
END
END

DECLARE @ObjectName sysname
DECLARE @Statement nvarchar(1024)

DECLARE curObjects INSENSITIVE CURSOR FOR
SELECT QUOTENAME( USER_NAME( uid ) ) + N'.' + QUOTENAME(name) FROM dbo.sysobjects
WHERE type = @objecttype AND status >= 0 ORDER BY name

OPEN curObjects
FETCH NEXT FROM curObjects INTO @ObjectName
WHILE (@@fetch_status = 0)
BEGIN
SELECT @Statement = @grantrevoke + N' ' + @permission + N' ON ' + @ObjectName + N' TO ' + @account
PRINT @Statement

EXEC ( @Statement )

FETCH NEXT FROM curObjects INTO @ObjectName
END
CLOSE curObjects
DEALLOCATE curObjects
RETURN 0
GO

GRANT EXECUTE ON sp__SetAllPermissions TO public
GO